#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>
#include "ib.h"

/* Init commands to the interface */
static char *init_cmd[] = {
   "auto 0",
   "eos 0",
   "eoi 1",
   "eor 3",
   "eot_enable 0",
   (void *)0};

/* ibcnt transfer counts */
volatile int ibcnt;
volatile long ibcntl;

/* ibsta status global and values */
volatile int ibsta;

/* iberr errorno global and values */
volatile int iberr;

/* internal state */
static int fd;
static char eos_char;
static int rd_eos;
static int wr_eoi;

/* minimal command set, enough to compile normal gpib code */
int
ibclr(int dev) {

   ibwrtcmd(0, "clr", strlen("clr"));
   return 0;
}

int
ibdev(int ignored, int dev, int sec_addr, int timeout_ms, int send_eoi, int eosmode) {
   struct termios tty;
   char buf[64];
   int i;

   if (!fd) fd = open("ttyGPIB", O_RDWR);
   if (fd < 0) return -1;

   if (tcgetattr(fd, &tty) != 0) return -1;
   tty.c_lflag &= ~(ICANON | ECHO | ECHOE | ECHONL | ISIG);
   tty.c_iflag &= ~(IXON | IXOFF | IXANY | IGNBRK | BRKINT | PARMRK | ISTRIP | INLCR | IGNCR | ICRNL);
   tty.c_oflag &= ~(OPOST | ONLCR);
   tty.c_cc[VTIME] = 2 + timeout_ms / 50; /* 200msec + about 3x as long as the board is set to */
   tty.c_cc[VMIN] = 0;   /* 0 min chars */
   tcsetattr(fd, TCSANOW, &tty);

   eos_char = eosmode & 0xff;
   rd_eos = eosmode & REOS;
   wr_eoi = send_eoi;

   for (i=0; init_cmd[i]; i++) ibwrtcmd(0, init_cmd[i], strlen(init_cmd[i]));
   sprintf(buf, "addr %d", dev);
   ibwrtcmd(0, buf, strlen(buf));
   sprintf(buf, "read_tmo_ms %d", timeout_ms),
   ibwrtcmd(0, buf, strlen(buf));
   if (!eos_char) ibwrtcmd(0, "eos 3", 5);
   if (wr_eoi)    ibwrtcmd(0, "eoi 1", 5);

   return 0;
}

int
ibonl(int ignored, int online) {
   return 0;
}

int
ibeos(int ignored, int eosmode) {
   eos_char = eosmode & 0xff;
   rd_eos = eosmode & REOS;
   if (!eos_char) ibwrtcmd(0, "eos 3", 5);
   else           ibwrtcmd(0, "eos 0", 5);

   return 0;
}

int
ibeot(int ignored, int send_eoi) {
   wr_eoi = send_eoi;
   if (wr_eoi) ibwrtcmd(0, "eoi 1", 5);
   else        ibwrtcmd(0, "eoi 0", 5);
   return 0;
}

int
ibfind(const char *ignored) {
   return 0;
}

int
ibrd(int ignored, void *buf, long len) {
   char rdcmd[16];
   int ret=0;
   int r;

   sprintf(rdcmd, rd_eos ? "read %d" : "read eoi", eos_char);
   ibwrtcmd(0, rdcmd, 7);

   while (ret != len) {
      r = read(fd, buf+ret, len-ret);
      if (r < 1) break;
      ret += r;
      if (rd_eos && ((char *)buf)[ret-1] == eos_char) break;
   }
   return ibcnt=ibcntl=ret;
}

int
ibrdcmd(int ignored, void *buf, long len) {
   int ret=0;
   int r;

   while (ret != len) {
      r = read(fd, buf+ret, len-ret);
      if (r < 1) break;
      ret += r;
      if (((char *)buf)[ret-1] == '\n') break;
   }
   return ret;
}

int
ibwrt(int ignored, const void *buf, long len) {
   char *esc_buf, *p, *q;
   int i;
   int esc_len = len+1;

   esc_buf = alloca(len*2+1);

   p = (char *)buf;
   q = esc_buf;
   for (i=0; i<len; i++) {
      if (*p == 0x0a || *p == 0x0d || *p == 0x1b || *p == '+') {
         *(q++) = 0x1b;
         esc_len++;
      }
      *(q++) = *(p++);
   }
   *(q++) = '\n'; /* terminator for AR488 */

   return ibcnt=ibcntl=write(fd, esc_buf, esc_len);
}

int
ibwrtcmd(int ignored, const void *buf, long len) {
   char *esc_buf, *p, *q;
   int i;
   int esc_len = len+3;

   esc_buf = alloca(len+3);

   p = (char *)buf;
   q = esc_buf;
   *(q++) = '+';
   *(q++) = '+';

   for (i=0; i<len; i++) {
      *(q++) = *(p++);
   }
   *(q++) = '\n';

   return write(fd, esc_buf, esc_len);
}

