/**************************************************************************
 *
 * This file is part of Buildbot.  Buildbot is free software: you can
 * redistribute it and/or modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation, version 2.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>

#include "tdsrom-gpib.h"
#include "tdsrom-cmds.h"

int read_memory(uint32_t addr, uint8_t *buf, int len)
{
	struct memory_read_cmd cmd;
	struct cmd_hdr hdr;

	memset(&cmd, 0, sizeof(cmd));
	cmd.hdr.cmd = 'm';
	cmd.hdr.len = cpu_to_be16(sizeof(cmd) - 4);
	cmd.addr = cpu_to_be32(addr);
	cmd.len = cpu_to_be32(len);

	return gpib_command((struct cmd_hdr *)&cmd, sizeof(struct memory_read_cmd), '=', &hdr, buf, len) > -1 ? 0:-1;
}

int write_memory(uint32_t addr, uint8_t *buf, int len)
{
	struct memory_write_cmd cmd;
	struct cmd_hdr hdr;

	memset(&cmd, 0, sizeof(cmd));
	cmd.hdr.cmd = 'M';
	cmd.hdr.len = cpu_to_be16(len + 8);
	cmd.addr = cpu_to_be32(addr);
	cmd.len = cpu_to_be32(len);

	memcpy(cmd.buf, buf, len);

	return gpib_command((struct cmd_hdr *)&cmd, len+12, '=', &hdr, (void *)0, 0) > -1 ? 0:-1;
}

int execute(uint32_t addr, uint32_t arg, uint8_t *buf, int len)
{
	struct branch_cmd cmd;
	struct cmd_hdr hdr;
	uint8_t rbuf[32+512];
	int rlen;

	memset(&cmd, 0, sizeof(cmd));
	cmd.hdr.cmd = 'B';
	cmd.hdr.len = cpu_to_be16(16+len);
	cmd.fcn  = cpu_to_be32(addr);
	cmd.arg0 = cpu_to_be32(arg);
	memcpy(cmd.data, buf, len);
	rlen=gpib_command((struct cmd_hdr *)&cmd, 20+len, '=', &hdr, rbuf, 512+20);
	if (rlen > 0) memcpy(buf, rbuf+16, rlen-16);

	return rlen > 0 ? rlen-16 : -1;
}
