/**************************************************************************
 *
 * This file is part of Buildbot.  Buildbot is free software: you can
 * redistribute it and/or modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation, version 2.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *
 **************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdint.h>
#include <string.h>
#include <sys/errno.h>

#include "tdsrom-gpib.h"

#include "ib.h"

extern int  Dev;
const char *ErrorMnemonic[] = {"EDVR", "ECIC", "ENOL", "EADR", "EARG",
			       "ESAC", "EABO", "ENEB", "EDMA", "",
			       "EOIP", "ECAP", "EFSO", "", "EBUS",
			       "ESTB", "ESRQ", "", "", "", "ETAB"};

extern int debug;

static void build_csum(struct cmd_hdr *hdr)
{
	uint8_t csum = 0;
	uint32_t i;
	for(i = 0; i < be16_to_cpu(hdr->len) + sizeof(struct cmd_hdr); i++)
			csum += ((uint8_t *)hdr)[i];
	hdr->csum = csum;
}

int gpib_command(struct cmd_hdr *cmd, int len, char ret, struct cmd_hdr *hdr, uint8_t *buf, int elen) {
	int loops = 0;
	char c;
	int rlen, i;
retry:
	build_csum(cmd);

	ibwrt (Dev, cmd, len);
// printf("len %d [", len);for (i=0; i< len; i++) printf("%.2x ", ((uint8_t *)cmd)[i]); printf("]\n");
	if (ibsta & ERR) {
		fprintf(stderr, "%s: writing command failed\n", __FUNCTION__);
		return -1;
	}

	ibrd(Dev, &c, 1);
	if (ibcntl != 1 || c != '+') {
		fprintf(stderr, "%s: ack reading failed\n", __FUNCTION__);
		if (loops++ > 10) return -1;
		ibrd(Dev, hdr, sizeof(struct cmd_hdr));
		goto retry;
	}

	for (loops=0; loops < 200; loops++) {
		ibrd(Dev, hdr, sizeof(struct cmd_hdr));
		if (ibsta & ERR) {
			fprintf(stderr, "%s: response reading failed\n", __FUNCTION__);
			return -1;
		}
		if (ibcntl == sizeof(struct cmd_hdr)) break;
		if (debug) { printf("."); fflush(stdout); }
	}
	if (loops && debug) printf("\n");
	if (ibcntl < sizeof(struct cmd_hdr)) {
		fprintf(stderr, "%s: short header\n", __FUNCTION__);
		return -1;
	}
	if (debug) {
		printf("read header\n");
		fprintf(stderr, "%s: response code: %c 0x%.2x len %d\n", __FUNCTION__, hdr->cmd, hdr->cmd, ((hdr->len & 0xff)<<8)||(hdr->len>>8));
	}
	if (ret && ret != hdr->cmd) return -1;

	rlen = be16_to_cpu(hdr->len);
	if (elen && rlen > elen) {
		fprintf(stderr, "return data is too long %d for buffer %d\n", rlen, elen);
		return -1;
	}

	if (elen) {
		ibrd(Dev, buf, rlen);
		if (debug) printf("read %ld data bytes\n", ibcntl);
		if (ibsta & ERR || ibcntl < rlen) {
			fprintf(stderr, "%s: return data reading failed\n", __FUNCTION__);
			return -1;
		}
	}
	c = '+';
	ibwrt(Dev, &c, 1);
usleep(30000);
ibclr(Dev);
	return rlen;
}
