#include "tds.h"

#define ARG_BASE  0x0f000000
#define ARG_BANK  0x00fc0000
#define ARG_ADDR  0x00fffffc
#define ARG_OP    0xf0000000

#define ARG_OP_PI 0x10000000
#define ARG_OP_ID 0x20000000
#define ARG_OP_PG 0x40000000
#define ARG_OP_ER 0x80000000

#define CMD8(X)  ( (((X)&0xff)<<24) | (((X)&0xff)<<16) | (((X)&0xff)<<8) | ((X)&0xff) )
#define CMD16(X) ( (((X)&0xffff)<<16) | ((X)&0xffff) )

#define OP_ID      0x90
#define OP_PROGRAM 0x40
#define OP_STATUS  0x70
#define OP_ERASE   0x20
#define OP_ERASE_2 0xd0
#define OP_RESET   0xff

#define STAT_RDY   0x80

volatile uint32_t *flash_base = (void *)0x01000000;

static inline
void td(int cnt)
{
   volatile int i;

   for (i=0; i<cnt; i++) { }
}

int
id_command(int arg, uint8_t *data, int len)
{
  volatile int i;
  uint32_t val;

  flash_base = (void *)(arg & ARG_BASE);

  flash_base[0] = CMD8(OP_RESET);
  td(100);

  flash_base[0] = CMD8(OP_ID);
  val = flash_base[0];
  for (i=0; i<4; i++) data[i]   = ((uint8_t *)(&val))[i];
  val = flash_base[1];
  for (i=0; i<4; i++) data[i+4] = ((uint8_t *)(&val))[i];
 
  flash_base[0] = CMD8(OP_RESET);
  flash_base[0] = CMD8(OP_STATUS);
  val = flash_base[0];
  for (i=0; i<4; i++) data[i+8] = ((uint8_t *)(&val))[i];

  flash_base[0] = CMD8(OP_RESET);

  return 12;
}

int
erase_command(int arg, uint8_t *data, int len)
{
  volatile int i;
  uint32_t val=0;
  uint32_t bank = (arg & ARG_BANK) >> 2; // long word bankess
  int loops = 60000;

  flash_base = (void *)(arg & ARG_BASE);

  flash_base[0] = CMD8(OP_RESET);
  td(100);

  flash_base[bank] = CMD8(OP_ERASE);
  flash_base[bank] = CMD8(OP_ERASE_2);

  while (loops && val != CMD8(STAT_RDY)) {
    td(1000);
    val = flash_base[bank] & CMD8(STAT_RDY);
  }

  flash_base[0] = CMD8(OP_RESET);
  flash_base[0] = CMD8(OP_STATUS);
  val = flash_base[0];
  for (i=0; i<4; i++) data[i] = ((uint8_t *)(&val))[i];

  flash_base[0] = CMD8(OP_RESET);

  return 4;
}

int
program_command(int arg, uint8_t *data, int len)
{
  volatile int i;
  uint32_t val=0;
  uint32_t addr = (arg & ARG_ADDR) >> 2; // long word address
  int loops = 6000;
  uint32_t *p = (void *)data;

  flash_base = (void *)(arg & ARG_BASE);

  flash_base[0] = CMD8(OP_RESET);
  td(100);

  len = ((len + 3) & ARG_ADDR)>>2; // word count
  for (i=0; i<len; i++) {
    flash_base[addr] = CMD8(OP_PROGRAM);
    flash_base[addr] = *p++;

    while (loops && val != CMD8(STAT_RDY)) {
      td(1000);
      val = flash_base[addr] & CMD8(STAT_RDY);
    }
    addr++;
  }

  flash_base[0] = CMD8(OP_RESET);
  flash_base[0] = CMD8(OP_STATUS);
  val = flash_base[0];
  for (i=0; i<4; i++) data[i] = ((uint8_t *)(&val))[i];

  flash_base[0] = CMD8(OP_RESET);

  return 4;
}

char version[] = "Target shim 0v1 for 28F008SA";

int
command(int arg, uint8_t *data, int len)
{
  int ret=0;

  if (arg & ARG_OP_PI) {
    for (ret=0; ret<sizeof(version); ret++) data[ret] = version[ret];
  }

  if (arg & ARG_OP_ID) ret = id_command(arg, data, len);
  if (arg & ARG_OP_ER) ret = erase_command(arg, data, len);
  if (arg & ARG_OP_PG) ret = program_command(arg, data, len);

  return ret;
}
